@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -17,7 +17,7 @@
 #include <crm/msg_xml.h>
 #include <crm/pengine/rules.h>
 #include <crm/cluster/internal.h>
-#include <crm/cluster/election.h>
+#include <crm/cluster/election_internal.h>
 #include <crm/common/ipc_internal.h>
 
 #include <pacemaker-controld.h>
@@ -131,6 +131,7 @@ extern char *max_generation_from;
 extern xmlNode *max_generation_xml;
 extern GHashTable *resource_history;
 extern GHashTable *voted;
+extern pcmk__output_t *logger_out;
 
 void
 crmd_fast_exit(crm_exit_t exit_code)
@@ -145,13 +146,20 @@ crmd_fast_exit(crm_exit_t exit_code)
         crm_err("Could not recover from internal error");
         exit_code = CRM_EX_ERROR;
     }
+
+    if (logger_out != NULL) {
+        logger_out->finish(logger_out, exit_code, true, NULL);
+        pcmk__output_free(logger_out);
+        logger_out = NULL;
+    }
+
     crm_exit(exit_code);
 }
 
 crm_exit_t
 crmd_exit(crm_exit_t exit_code)
 {
-    GListPtr gIter = NULL;
+    GList *gIter = NULL;
     GMainLoop *mloop = crmd_mainloop;
 
     static bool in_progress = FALSE;
@@ -183,7 +191,7 @@ crmd_exit(crm_exit_t exit_code)
     }
 
     controld_close_attrd_ipc();
-    pe_subsystem_free();
+    controld_shutdown_schedulerd_ipc();
     controld_disconnect_fencer(TRUE);
 
     if ((exit_code == CRM_EX_OK) && (crmd_mainloop == NULL)) {
@@ -225,10 +233,8 @@ crmd_exit(crm_exit_t exit_code)
     /* Tear down the CIB manager connection, but don't free it yet -- it could
      * be used when we drain the mainloop later.
      */
-    fsa_cib_conn->cmds->del_notify_callback(fsa_cib_conn, T_CIB_REPLACE_NOTIFY, do_cib_replaced);
-    fsa_cib_conn->cmds->del_notify_callback(fsa_cib_conn, T_CIB_DIFF_NOTIFY, do_cib_updated);
-    cib_free_callbacks(fsa_cib_conn);
-    fsa_cib_conn->cmds->signoff(fsa_cib_conn);
+
+    controld_disconnect_cib_manager();
 
     verify_stopped(fsa_state, LOG_WARNING);
     controld_clear_fsa_input_flags(R_LRM_CONNECTED);
@@ -384,14 +390,13 @@ dispatch_controller_ipc(qb_ipcs_connection_t * c, void *data, size_t size)
     }
     pcmk__ipc_send_ack(client, id, flags, "ack", CRM_EX_INDETERMINATE);
 
-#if ENABLE_ACL
     CRM_ASSERT(client->user != NULL);
     pcmk__update_acl_user(msg, F_CRM_USER, client->user);
-#endif
 
     crm_xml_add(msg, F_CRM_SYS_FROM, client->id);
     if (controld_authorize_ipc_message(msg, client, NULL)) {
-        crm_trace("Processing IPC message from %s", pcmk__client_name(client));
+        crm_trace("Processing IPC message from client %s",
+                  pcmk__client_name(client));
         route_message(C_IPC_MESSAGE, msg);
     }
 
@@ -516,7 +521,7 @@ static pcmk__cluster_option_t crmd_opts[] = {
      * long description
      */
     {
-        "dc-version", NULL, "string", NULL, "none", NULL,
+        "dc-version", NULL, "string", NULL, PCMK__VALUE_NONE, NULL,
         "Pacemaker version on cluster node elected Designated Controller (DC)",
         "Includes a hash which identifies the exact changeset the code was "
             "built from. Used for diagnostic purposes."
@@ -543,8 +548,8 @@ static pcmk__cluster_option_t crmd_opts[] = {
     },
     {
         XML_CONFIG_ATTR_RECHECK, NULL, "time",
-        "Zero disables polling, while positive values are an interval in seconds"
-            "(unless other units are specified, for example \"5min\")",
+        N_("Zero disables polling, while positive values are an interval in seconds"
+            "(unless other units are specified, for example \"5min\")"),
         "15min", pcmk__valid_interval_spec,
         "Polling interval to recheck cluster state and evaluate rules "
             "with date specifications",
@@ -556,7 +561,7 @@ static pcmk__cluster_option_t crmd_opts[] = {
     },
     {
         "load-threshold", NULL, "percentage", NULL,
-        "80%", pcmk__valid_utilization,
+        "80%", pcmk__valid_percentage,
         "Maximum amount of system load that should be used by cluster nodes",
         "The cluster will slow down its recovery process when the amount of "
             "system resources used (currently CPU) approaches this limit",
@@ -572,7 +577,7 @@ static pcmk__cluster_option_t crmd_opts[] = {
         "A cluster node may receive notification of its own fencing if fencing "
         "is misconfigured, or if fabric fencing is in use that doesn't cut "
         "cluster communication. Allowed values are \"stop\" to attempt to "
-        "immediately stop pacemaker and stay stopped, or \"panic\" to attempt "
+        "immediately stop Pacemaker and stay stopped, or \"panic\" to attempt "
         "to immediately reboot the local node, falling back to stop on failure."
     },
     {
@@ -616,7 +621,7 @@ static pcmk__cluster_option_t crmd_opts[] = {
     },
     {
         "stonith-watchdog-timeout", NULL, "time", NULL,
-        "0", pcmk__valid_sbd_timeout,
+        "0", controld_verify_stonith_watchdog_timeout,
         "How long to wait before we can assume nodes are safely down "
             "when watchdog-based self-fencing via SBD is in use",
         "If nonzero, along with `have-watchdog=true` automatically set by the "
@@ -634,7 +639,7 @@ static pcmk__cluster_option_t crmd_opts[] = {
             "If `stonith-watchdog-timeout` is set to a negative value, and "
             "`SBD_WATCHDOG_TIMEOUT` is set, twice that value will be used. "
             "+WARNING:+ In this case, it's essential (currently not verified by "
-            "pacemaker) that `SBD_WATCHDOG_TIMEOUT` is set to the same value on "
+            "Pacemaker) that `SBD_WATCHDOG_TIMEOUT` is set to the same value on "
             "all nodes."
     },
     {
@@ -646,7 +651,7 @@ static pcmk__cluster_option_t crmd_opts[] = {
 
     // Already documented in libpe_status (other values must be kept identical)
     {
-        "no-quorum-policy", NULL, "enum", "stop, freeze, ignore, demote, suicide",
+        "no-quorum-policy", NULL, "select", "stop, freeze, ignore, demote, suicide",
         "stop", pcmk__valid_quorum, NULL, NULL
     },
     {
@@ -658,23 +663,26 @@ static pcmk__cluster_option_t crmd_opts[] = {
 void
 crmd_metadata(void)
 {
-    pcmk__print_option_metadata("pacemaker-controld", "1.0",
-                                "Pacemaker controller options",
-                                "Cluster options used by Pacemaker's "
-                                    "controller (formerly called crmd)",
-                                crmd_opts, DIMOF(crmd_opts));
+    char *s = pcmk__format_option_metadata("pacemaker-controld",
+                                           "Pacemaker controller options",
+                                           "Cluster options used by Pacemaker's "
+                                               "controller",
+                                           crmd_opts, PCMK__NELEM(crmd_opts));
+    printf("%s", s);
+    free(s);
 }
 
 static void
 verify_crmd_options(GHashTable * options)
 {
-    pcmk__validate_cluster_options(options, crmd_opts, DIMOF(crmd_opts));
+    pcmk__validate_cluster_options(options, crmd_opts, PCMK__NELEM(crmd_opts));
 }
 
 static const char *
 crmd_pref(GHashTable * options, const char *name)
 {
-    return pcmk__cluster_option(options, crmd_opts, DIMOF(crmd_opts), name);
+    return pcmk__cluster_option(options, crmd_opts, PCMK__NELEM(crmd_opts),
+                                name);
 }
 
 static void
@@ -714,7 +722,7 @@ config_query_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void
     }
 
     crm_debug("Call %d : Parsing CIB options", call_id);
-    config_hash = crm_str_table_new();
+    config_hash = pcmk__strkey_table(free, free);
     pe_unpack_nvpairs(crmconfig, crmconfig, XML_CIB_TAG_PROPSET, NULL,
                       config_hash, CIB_OPTIONS_FIRST, FALSE, now, NULL);
 
