@@ -1,5 +1,5 @@
 /*
- * Copyright 2009-2020 the Pacemaker project contributors
+ * Copyright 2009-2022 the Pacemaker project contributors
  *
  * This source code is licensed under the GNU General Public License version 2
  * or later (GPLv2+) WITHOUT ANY WARRANTY.
@@ -10,13 +10,14 @@
 
 /*!
  * \internal
- * \brief Check to see if target was fenced in the last few seconds.
- * \param tolerance, The number of seconds to look back in time
- * \param target, The node to search for
- * \param action, The action we want to match.
+ * \brief Check whether target has already been fenced recently
  *
- * \retval FALSE, not match
- * \retval TRUE, fencing operation took place in the last 'tolerance' number of seconds.
+ * \param[in] tolerance  Number of seconds to look back in time
+ * \param[in] target     Name of node to search for
+ * \param[in] action     Action we want to match
+ *
+ * \return TRUE if an equivalent fencing operation took place in the last
+ *         \p tolerance seconds, FALSE otherwise
  */
 gboolean stonith_check_fence_tolerance(int tolerance, const char *target, const char *action);
 
@@ -27,7 +28,7 @@ typedef struct stonith_device_s {
 
     /*! list of actions that must execute on the target node. Used for unfencing */
     char *on_target_actions;
-    GListPtr targets;
+    GList *targets;
     time_t targets_age;
     gboolean has_attr_map;
     /* should nodeid parameter for victim be included in agent arguments */
@@ -41,6 +42,7 @@ typedef struct stonith_device_s {
     GHashTable *params;
     GHashTable *aliases;
     GList *pending_ops;
+    mainloop_timer_t *timer;
     crm_trigger_t *work;
     xmlNode *agent_metadata;
 
@@ -50,6 +52,7 @@ typedef struct stonith_device_s {
 
     gboolean cib_registered;
     gboolean api_registered;
+    gboolean dirty;
 } stonith_device_t;
 
 /* These values are used to index certain arrays by "phase". Usually an
@@ -64,14 +67,6 @@ enum st_remap_phase {
     st_phase_max = 3
 };
 
-/* These values provide additional information for STONITH's asynchronous reply response.
- * The st_reply_opt_merged value indicates an operation that has been merged and completed without being executed.
- */
-enum st_replay_option {
-    st_reply_opt_none            = 0x00000000,
-    st_reply_opt_merged          = 0x00000001,
-};
-
 typedef struct remote_fencing_op_s {
     /* The unique id associated with this operation */
     char *id;
@@ -133,7 +128,7 @@ typedef struct remote_fencing_op_s {
     /*! The name of client that initiated the fencing request */
     char *client_name;
     /*! List of the received query results for all the nodes in the cpg group */
-    GListPtr query_results;
+    GList *query_results;
     /*! The original request that initiated the remote stonith operation */
     xmlNode *request;
 
@@ -143,25 +138,24 @@ typedef struct remote_fencing_op_s {
     enum st_remap_phase phase;
 
     /*! Devices with automatic unfencing (always run if "on" requested, never if remapped) */
-    GListPtr automatic_list;
+    GList *automatic_list;
     /*! List of all devices at the currently executing topology level */
-    GListPtr devices_list;
+    GList *devices_list;
     /*! Current entry in the topology device list */
-    GListPtr devices;
+    GList *devices;
 
     /*! List of duplicate operations attached to this operation. Once this operation
      * completes, the duplicate operations will be closed out as well. */
-    GListPtr duplicates;
+    GList *duplicates;
 
+    /*! The point at which the remote operation completed(nsec) */
+    long long completed_nsec;
+
+    /*! The (potentially intermediate) result of the operation */
+    pcmk__action_result_t result;
 } remote_fencing_op_t;
 
-/*!
- * \internal
- * \brief Broadcast the result of an operation to the peers.
- * \param op, Operation whose result should be broadcast
- * \param rc, Result of the operation
- */
-void stonith_bcast_result_to_peers(remote_fencing_op_t * op, int rc, gboolean op_merged);
+void fenced_broadcast_op_result(remote_fencing_op_t *op, bool op_merged);
 
 // Fencer-specific client flags
 enum st_client_flags {
@@ -173,6 +167,14 @@ enum st_client_flags {
     st_callback_notify_history_synced = (UINT64_C(1) << 6)
 };
 
+// How the user specified the target of a topology level
+enum fenced_target_by {
+    fenced_target_by_unknown = -1,  // Invalid or not yet parsed
+    fenced_target_by_name,          // By target name
+    fenced_target_by_pattern,       // By a pattern matching target names
+    fenced_target_by_attribute,     // By a node attribute/value on target
+};
+
 /*
  * Complex fencing requirements are specified via fencing topologies.
  * A topology consists of levels; each level is a list of fencing devices.
@@ -188,7 +190,7 @@ enum st_client_flags {
  * Topology levels start from 1, so levels[0] is unused and always NULL.
  */
 typedef struct stonith_topology_s {
-    int kind;
+    enum fenced_target_by kind; // How target was specified
 
     /*! Node name regex or attribute name=value for which topology applies */
     char *target;
@@ -197,7 +199,7 @@ typedef struct stonith_topology_s {
     char *target_attribute;
 
     /*! Names of fencing devices at each topology level */
-    GListPtr levels[ST_LEVEL_MAX];
+    GList *levels[ST_LEVEL_MAX];
 
 } stonith_topology_t;
 
@@ -208,6 +210,7 @@ void free_topology_list(void);
 void free_stonith_remote_op_list(void);
 void init_stonith_remote_op_hash_table(GHashTable **table);
 void free_metadata_cache(void);
+void fenced_unregister_handlers(void);
 
 uint64_t get_stonith_flag(const char *name);
 
@@ -216,56 +219,72 @@ void stonith_command(pcmk__client_t *client, uint32_t id, uint32_t flags,
 
 int stonith_device_register(xmlNode * msg, const char **desc, gboolean from_cib);
 
-int stonith_device_remove(const char *id, gboolean from_cib);
+void stonith_device_remove(const char *id, bool from_cib);
 
 char *stonith_level_key(xmlNode * msg, int mode);
-int stonith_level_kind(xmlNode * msg);
-int stonith_level_register(xmlNode * msg, char **desc);
-
-int stonith_level_remove(xmlNode * msg, char **desc);
+void fenced_register_level(xmlNode *msg, char **desc,
+                           pcmk__action_result_t *result);
+void fenced_unregister_level(xmlNode *msg, char **desc,
+                             pcmk__action_result_t *result);
 
 stonith_topology_t *find_topology_for_host(const char *host);
 
-void do_local_reply(xmlNode * notify_src, const char *client_id, gboolean sync_reply,
-                           gboolean from_peer);
+void do_local_reply(xmlNode *notify_src, pcmk__client_t *client,
+                    int call_options);
 
-xmlNode *stonith_construct_reply(xmlNode * request, const char *output, xmlNode * data,
-                                        int rc);
+xmlNode *fenced_construct_reply(xmlNode *request, xmlNode *data,
+                                pcmk__action_result_t *result);
 
 void
  do_stonith_async_timeout_update(const char *client, const char *call_id, int timeout);
 
-void do_stonith_notify(int options, const char *type, int result, xmlNode * data);
-void do_stonith_notify_device(int options, const char *op, int rc, const char *desc);
-void do_stonith_notify_level(int options, const char *op, int rc, const char *desc);
+void fenced_send_notification(const char *type,
+                              const pcmk__action_result_t *result,
+                              xmlNode *data);
+void fenced_send_device_notification(const char *op,
+                                     const pcmk__action_result_t *result,
+                                     const char *desc);
+void fenced_send_level_notification(const char *op,
+                                    const pcmk__action_result_t *result,
+                                    const char *desc);
 
 remote_fencing_op_t *initiate_remote_stonith_op(pcmk__client_t *client,
                                                 xmlNode *request,
                                                 gboolean manual_ack);
 
-int process_remote_stonith_exec(xmlNode * msg);
+void fenced_process_fencing_reply(xmlNode *msg);
 
 int process_remote_stonith_query(xmlNode * msg);
 
 void *create_remote_stonith_op(const char *client, xmlNode * request, gboolean peer);
 
-int stonith_fence_history(xmlNode *msg, xmlNode **output,
-                          const char *remote_peer, int options);
+void stonith_fence_history(xmlNode *msg, xmlNode **output,
+                           const char *remote_peer, int options);
 
 void stonith_fence_history_trim(void);
 
 bool fencing_peer_active(crm_node_t *peer);
 
-int stonith_manual_ack(xmlNode * msg, remote_fencing_op_t * op);
+void set_fencing_completed(remote_fencing_op_t * op);
 
-gboolean string_in_list(GListPtr list, const char *item);
+int fenced_handle_manual_confirmation(pcmk__client_t *client, xmlNode *msg);
 
 gboolean node_has_attr(const char *node, const char *name, const char *value);
 
+gboolean node_does_watchdog_fencing(const char *node);
+
+static inline void
+fenced_set_protocol_error(pcmk__action_result_t *result)
+{
+    pcmk__set_result(result, CRM_EX_PROTOCOL, PCMK_EXEC_INVALID,
+                     "Fencer API request missing required information (bug?)");
+}
+
 extern char *stonith_our_uname;
 extern gboolean stand_alone;
 extern GHashTable *device_list;
 extern GHashTable *topology;
 extern long stonith_watchdog_timeout_ms;
+extern GList *stonith_watchdog_targets;
 
 extern GHashTable *stonith_remote_op_list;
