@@ -6,8 +6,8 @@
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
-#ifndef PCMKI_RESOURCE__H
-#define PCMKI_RESOURCE__H
+#ifndef PCMK__PCMKI_PCMKI_RESOURCE__H
+#define PCMK__PCMKI_PCMKI_RESOURCE__H
 
 #include <glib.h>
 
@@ -18,4 +18,4 @@ int pcmk__resource_digests(pcmk__output_t *out, pe_resource_t *rsc,
                            pe_node_t *node, GHashTable *overrides,
                            pe_working_set_t *data_set);
 
-#endif /* PCMK_RESOURCE__H */
+#endif /* PCMK__PCMKI_PCMKI_RESOURCE__H */
