@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2021 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -769,6 +769,34 @@ pcmk__ipc_send_xml(pcmk__client_t *c, uint32_t request, xmlNode *message,
     return rc;
 }
 
+/*!
+ * \internal
+ * \brief Create an acknowledgement with a status code to send to a client
+ *
+ * \param[in] function  Calling function
+ * \param[in] line      Source file line within calling function
+ * \param[in] flags     IPC flags to use when sending
+ * \param[in] tag       Element name to use for acknowledgement
+ * \param[in] status    Exit status code to add to ack
+ *
+ * \return Newly created XML for ack
+ * \note The caller is responsible for freeing the return value with free_xml().
+ */
+xmlNode *
+pcmk__ipc_create_ack_as(const char *function, int line, uint32_t flags,
+                        const char *tag, crm_exit_t status)
+{
+    xmlNode *ack = NULL;
+
+    if (pcmk_is_set(flags, crm_ipc_client_response)) {
+        ack = create_xml_node(NULL, tag);
+        crm_xml_add(ack, "function", function);
+        crm_xml_add_int(ack, "line", line);
+        crm_xml_add_int(ack, "status", (int) status);
+    }
+    return ack;
+}
+
 /*!
  * \internal
  * \brief Send an acknowledgement with a status code to a client
@@ -777,7 +805,6 @@ pcmk__ipc_send_xml(pcmk__client_t *c, uint32_t request, xmlNode *message,
  * \param[in] line      Source file line within calling function
  * \param[in] c         Client to send ack to
  * \param[in] request   Request ID being replied to
- * \param[in] status    Exit status code to add to ack
  * \param[in] flags     IPC flags to use when sending
  * \param[in] tag       Element name to use for acknowledgement
  * \param[in] status    Status code to send with acknowledgement
@@ -790,16 +817,12 @@ pcmk__ipc_send_ack_as(const char *function, int line, pcmk__client_t *c,
                       crm_exit_t status)
 {
     int rc = pcmk_rc_ok;
+    xmlNode *ack = pcmk__ipc_create_ack_as(function, line, flags, tag, status);
 
-    if (pcmk_is_set(flags, crm_ipc_client_response)) {
-        xmlNode *ack = create_xml_node(NULL, tag);
-
+    if (ack != NULL) {
         crm_trace("Ack'ing IPC message from client %s as <%s status=%d>",
                   pcmk__client_name(c), tag, status);
         c->request_id = 0;
-        crm_xml_add(ack, "function", function);
-        crm_xml_add_int(ack, "line", line);
-        crm_xml_add_int(ack, "status", (int) status);
         rc = pcmk__ipc_send_xml(c, request, ack, flags);
         free_xml(ack);
     }
@@ -944,6 +967,20 @@ pcmk__serve_pacemakerd_ipc(qb_ipcs_service_t **ipcs,
     }
 }
 
+/*!
+ * \internal
+ * \brief Add an IPC server to the main loop for the pacemaker-schedulerd API
+ *
+ * \param[in] cb IPC callbacks
+ *
+ * \note This function exits fatally if unable to create the servers.
+ */
+qb_ipcs_service_t *
+pcmk__serve_schedulerd_ipc(struct qb_ipcs_service_handlers *cb)
+{
+    return mainloop_add_ipc_server(CRM_SYSTEM_PENGINE, QB_IPC_NATIVE, cb);
+}
+
 /*!
  * \brief Check whether string represents a client name used by cluster daemons
  *
