@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -33,6 +33,15 @@
 #include <crm/crm.h>
 #include <crm/common/mainloop.h>
 
+// Use high-resolution (millisecond) timestamps if libqb supports them
+#ifdef QB_FEATURE_LOG_HIRES_TIMESTAMPS
+#define TIMESTAMP_FORMAT_SPEC "%%T"
+typedef struct timespec *log_time_t;
+#else
+#define TIMESTAMP_FORMAT_SPEC "%%t"
+typedef time_t log_time_t;
+#endif
+
 unsigned int crm_log_level = LOG_INFO;
 unsigned int crm_trace_nonlog = 0;
 bool pcmk__is_daemon = false;
@@ -122,8 +131,21 @@ crm_log_deinit(void)
 
 #define FMT_MAX 256
 
+/*!
+ * \internal
+ * \brief Set the log format string based on the passed-in method
+ *
+ * \param[in] method        The detail level of the log output
+ * \param[in] daemon        The daemon ID included in error messages
+ * \param[in] use_pid       Cached result of getpid() call, for efficiency
+ * \param[in] use_nodename  Cached result of uname() call, for efficiency
+ *
+ */
+
+/* XXX __attribute__((nonnull)) for use_nodename parameter */
 static void
-set_format_string(int method, const char *daemon)
+set_format_string(int method, const char *daemon, pid_t use_pid,
+                  const char *use_nodename)
 {
     if (method == QB_LOG_SYSLOG) {
         // The system log gets a simplified, user-friendly format
@@ -137,17 +159,10 @@ set_format_string(int method, const char *daemon)
         char fmt[FMT_MAX];
 
         if (method > QB_LOG_STDERR) {
-            struct utsname res;
-            const char *nodename = "localhost";
-
-            if (uname(&res) == 0) {
-                nodename = res.nodename;
-            }
-
             // If logging to file, prefix with timestamp, node name, daemon ID
             offset += snprintf(fmt + offset, FMT_MAX - offset,
-                               "%%t %s %-20s[%lu] ",
-                               nodename, daemon, (unsigned long) getpid());
+                               TIMESTAMP_FORMAT_SPEC " %s %-20s[%lu] ",
+                                use_nodename, daemon, (unsigned long) use_pid);
         }
 
         // Add function name (in parentheses)
@@ -168,160 +183,209 @@ set_format_string(int method, const char *daemon)
 
 #define DEFAULT_LOG_FILE CRM_LOG_DIR "/pacemaker.log"
 
-gboolean
-crm_add_logfile(const char *filename)
+static bool
+logfile_disabled(const char *filename)
 {
-    bool is_default = false;
-    static int default_fd = -1;
-    static gboolean have_logfile = FALSE;
+    return pcmk__str_eq(filename, PCMK__VALUE_NONE, pcmk__str_casei)
+           || pcmk__str_eq(filename, "/dev/null", pcmk__str_none);
+}
 
-    struct stat parent;
-    int fd = 0, rc = 0;
-    FILE *logfile = NULL;
-    char *parent_dir = NULL;
-    char *filename_cp;
+/*!
+ * \internal
+ * \brief Fix log file ownership if group is wrong or doesn't have access
+ *
+ * \param[in] filename  Log file name (for logging only)
+ * \param[in] logfd     Log file descriptor
+ *
+ * \return Standard Pacemaker return code
+ */
+static int
+chown_logfile(const char *filename, int logfd)
+{
+    uid_t pcmk_uid = 0;
+    gid_t pcmk_gid = 0;
+    struct stat st;
+    int rc;
 
-    if (filename == NULL && have_logfile == FALSE) {
-        filename = DEFAULT_LOG_FILE;
+    // Get the log file's current ownership and permissions
+    if (fstat(logfd, &st) < 0) {
+        return errno;
     }
 
-    if ((filename == NULL)
-        || pcmk__str_eq(filename, "none", pcmk__str_casei)
-        || pcmk__str_eq(filename, "/dev/null", pcmk__str_none)) {
-        return FALSE; // User doesn't want logging, so there's nothing to do
+    // Any other errors don't prevent file from being used as log
 
-    } else if (pcmk__str_eq(filename, DEFAULT_LOG_FILE, pcmk__str_none)) {
-        is_default = TRUE;
+    rc = pcmk_daemon_user(&pcmk_uid, &pcmk_gid);
+    if (rc != pcmk_ok) {
+        rc = pcmk_legacy2rc(rc);
+        crm_warn("Not changing '%s' ownership because user information "
+                 "unavailable: %s", filename, pcmk_rc_str(rc));
+        return pcmk_rc_ok;
     }
-
-    if(is_default && default_fd >= 0) {
-        return TRUE;           /* Nothing to do */
+    if ((st.st_gid == pcmk_gid)
+        && ((st.st_mode & S_IRWXG) == (S_IRGRP|S_IWGRP))) {
+        return pcmk_rc_ok;
     }
-
-    /* Check the parent directory */
-    filename_cp = strdup(filename);
-    parent_dir = dirname(filename_cp);
-    rc = stat(parent_dir, &parent);
-
-    if (rc != 0) {
-        crm_err("Directory '%s' does not exist: logging to '%s' is disabled", parent_dir, filename);
-        free(filename_cp);
-        return FALSE;
+    if (fchown(logfd, pcmk_uid, pcmk_gid) < 0) {
+        crm_warn("Couldn't change '%s' ownership to user %s gid %d: %s",
+             filename, CRM_DAEMON_USER, pcmk_gid, strerror(errno));
     }
-    free(filename_cp);
+    return pcmk_rc_ok;
+}
 
-    errno = 0;
-    logfile = fopen(filename, "a");
-    if(logfile == NULL) {
-        crm_err("%s (%d): Logging to '%s' as uid=%u, gid=%u is disabled",
-                pcmk_strerror(errno), errno, filename, geteuid(), getegid());
-        return FALSE;
+// Reset log file permissions (using environment variable if set)
+static void
+chmod_logfile(const char *filename, int logfd)
+{
+    const char *modestr = getenv("PCMK_logfile_mode");
+    mode_t filemode = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP;
+
+    if (modestr != NULL) {
+        long filemode_l = strtol(modestr, NULL, 8);
+
+        if ((filemode_l != LONG_MIN) && (filemode_l != LONG_MAX)) {
+            filemode = (mode_t) filemode_l;
+        }
     }
+    if ((filemode != 0) && (fchmod(logfd, filemode) < 0)) {
+        crm_warn("Couldn't change '%s' mode to %04o: %s",
+                 filename, filemode, strerror(errno));
+    }
+}
 
-    /* Check/Set permissions if we're root */
+// If we're root, correct a log file's permissions if needed
+static int
+set_logfile_permissions(const char *filename, FILE *logfile)
+{
     if (geteuid() == 0) {
-        struct stat st;
-        uid_t pcmk_uid = 0;
-        gid_t pcmk_gid = 0;
-        gboolean fix = FALSE;
         int logfd = fileno(logfile);
-        const char *modestr;
-        mode_t filemode = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP;
+        int rc = chown_logfile(filename, logfd);
 
-        rc = fstat(logfd, &st);
-        if (rc < 0) {
-            crm_perror(LOG_WARNING, "Cannot stat %s", filename);
-            fclose(logfile);
-            return FALSE;
-        }
-
-        if (pcmk_daemon_user(&pcmk_uid, &pcmk_gid) == 0) {
-            if (st.st_gid != pcmk_gid) {
-                /* Wrong group */
-                fix = TRUE;
-            } else if ((st.st_mode & S_IRWXG) != (S_IRGRP | S_IWGRP)) {
-                /* Not read/writable by the correct group */
-                fix = TRUE;
-            }
-        }
-
-        modestr = getenv("PCMK_logfile_mode");
-        if (modestr) {
-            long filemode_l = strtol(modestr, NULL, 8);
-            if (filemode_l != LONG_MIN && filemode_l != LONG_MAX) {
-                filemode = (mode_t)filemode_l;
-	    }
-        }
-
-        if (fix) {
-            rc = fchown(logfd, pcmk_uid, pcmk_gid);
-            if (rc < 0) {
-                crm_warn("Cannot change the ownership of %s to user %s and gid %d",
-                         filename, CRM_DAEMON_USER, pcmk_gid);
-            }
-	}
-
-	if (filemode) {
-            rc = fchmod(logfd, filemode);
-            if (rc < 0) {
-                crm_warn("Cannot change the mode of %s to %o", filename, filemode);
-            }
-
-            fprintf(logfile, "Set r/w permissions for uid=%d, gid=%d on %s\n",
-                    pcmk_uid, pcmk_gid, filename);
-            if (fflush(logfile) < 0 || fsync(logfd) < 0) {
-                crm_err("Couldn't write out logfile: %s", filename);
-            }
+        if (rc != pcmk_rc_ok) {
+            return rc;
         }
+        chmod_logfile(filename, logfd);
     }
+    return pcmk_rc_ok;
+}
 
-    /* Close and reopen with libqb */
-    fclose(logfile);
-    fd = qb_log_file_open(filename);
-
-    if (fd < 0) {
-        crm_perror(LOG_WARNING, "Couldn't send additional logging to %s", filename);
-        return FALSE;
-    }
-
-    if(is_default) {
-        default_fd = fd;
-
-        // Some resource agents will log only if environment variable is set
-        if (pcmk__env_option("logfile") == NULL) {
-            pcmk__set_env_option("logfile", filename);
-        }
-
-    } else if(default_fd >= 0) {
-        crm_notice("Switching to %s", filename);
-        qb_log_ctl(default_fd, QB_LOG_CONF_ENABLED, QB_FALSE);
-    }
-
-    crm_notice("Additional logging available in %s", filename);
+// Enable libqb logging to a new log file
+static void
+enable_logfile(int fd)
+{
     qb_log_ctl(fd, QB_LOG_CONF_ENABLED, QB_TRUE);
-    /* qb_log_ctl(fd, QB_LOG_CONF_FILE_SYNC, 1);  Turn on synchronous writes */
+#if 0
+    qb_log_ctl(fd, QB_LOG_CONF_FILE_SYNC, 1); // Turn on synchronous writes
+#endif
 
 #ifdef HAVE_qb_log_conf_QB_LOG_CONF_MAX_LINE_LEN
     // Longer than default, for logging long XML lines
     qb_log_ctl(fd, QB_LOG_CONF_MAX_LINE_LEN, 800);
 #endif
 
-    /* Enable callsites */
     crm_update_callsites();
-    have_logfile = TRUE;
+}
 
-    return TRUE;
+static inline void
+disable_logfile(int fd)
+{
+    qb_log_ctl(fd, QB_LOG_CONF_ENABLED, QB_FALSE);
+}
+
+static void
+setenv_logfile(const char *filename)
+{
+    // Some resource agents will log only if environment variable is set
+    if (pcmk__env_option(PCMK__ENV_LOGFILE) == NULL) {
+        pcmk__set_env_option(PCMK__ENV_LOGFILE, filename);
+    }
+}
+
+/*!
+ * \brief Add a file to be used as a Pacemaker detail log
+ *
+ * \param[in] filename  Name of log file to use
+ *
+ * \return Standard Pacemaker return code
+ */
+int
+pcmk__add_logfile(const char *filename)
+{
+    /* No log messages from this function will be logged to the new log!
+     * If another target such as syslog has already been added, the messages
+     * should show up there.
+     */
+
+    int fd = 0;
+    int rc = pcmk_rc_ok;
+    FILE *logfile = NULL;
+    bool is_default = false;
+
+    static int default_fd = -1;
+    static bool have_logfile = false;
+
+    // Use default if caller didn't specify (and we don't already have one)
+    if (filename == NULL) {
+        if (have_logfile) {
+            return pcmk_rc_ok;
+        }
+        filename = DEFAULT_LOG_FILE;
+    }
+
+    // If the user doesn't want logging, we're done
+    if (logfile_disabled(filename)) {
+        return pcmk_rc_ok;
+    }
+
+    // If the caller wants the default and we already have it, we're done
+    is_default = pcmk__str_eq(filename, DEFAULT_LOG_FILE, pcmk__str_none);
+    if (is_default && (default_fd >= 0)) {
+        return pcmk_rc_ok;
+    }
+
+    // Check whether we have write access to the file
+    logfile = fopen(filename, "a");
+    if (logfile == NULL) {
+        rc = errno;
+        crm_warn("Logging to '%s' is disabled: %s " CRM_XS " uid=%u gid=%u",
+                 filename, strerror(rc), geteuid(), getegid());
+        return rc;
+    }
+
+    rc = set_logfile_permissions(filename, logfile);
+    if (rc != pcmk_rc_ok) {
+        crm_warn("Logging to '%s' is disabled: %s " CRM_XS " permissions",
+                 filename, strerror(rc));
+        fclose(logfile);
+        return rc;
+    }
+
+    // Close and reopen as libqb logging target
+    fclose(logfile);
+    fd = qb_log_file_open(filename);
+    if (fd < 0) {
+        crm_warn("Logging to '%s' is disabled: %s " CRM_XS " qb_log_file_open",
+                 filename, strerror(-fd));
+        return -fd; // == +errno
+    }
+
+    if (is_default) {
+        default_fd = fd;
+        setenv_logfile(filename);
+
+    } else if (default_fd >= 0) {
+        crm_notice("Switching logging to %s", filename);
+        disable_logfile(default_fd);
+    }
+
+    crm_notice("Additional logging available in %s", filename);
+    enable_logfile(fd);
+    have_logfile = true;
+    return pcmk_rc_ok;
 }
 
 static int blackbox_trigger = 0;
 static volatile char *blackbox_file_prefix = NULL;
 
-#ifdef QB_FEATURE_LOG_HIRES_TIMESTAMPS
-typedef struct timespec *log_time_t;
-#else
-typedef time_t log_time_t;
-#endif
-
 static void
 blackbox_logger(int32_t t, struct qb_log_callsite *cs, log_time_t timestamp,
                 const char *msg)
@@ -463,12 +527,6 @@ crm_write_blackbox(int nsig, struct qb_log_callsite *cs)
     }
 }
 
-gboolean
-crm_log_cli_init(const char *entity)
-{
-    return crm_log_init(entity, LOG_ERR, FALSE, FALSE, 0, NULL, TRUE);
-}
-
 static const char *
 crm_quark_to_string(uint32_t tag)
 {
@@ -658,32 +716,48 @@ crm_priority2int(const char *name)
 }
 
 
+/*!
+ * \internal
+ * \brief Set the identifier for the current process
+ *
+ * If the identifier crm_system_name is not already set, then it is set as follows:
+ * - it is passed to the function via the "entity" parameter, or
+ * - it is derived from the executable name
+ * 
+ * The identifier can be used in logs, IPC, and more.
+ * 
+ * This method also sets the PCMK_service environment variable.
+ *
+ * \param[in] entity  If not NULL, will be assigned to the identifier
+ * \param[in] argc    The number of command line parameters
+ * \param[in] argv    The command line parameter values
+ */
 static void
-crm_identity(const char *entity, int argc, char **argv) 
+set_identity(const char *entity, int argc, char **argv)
 {
-    if(crm_system_name != NULL) {
-        /* Nothing to do */
+    if (crm_system_name != NULL) {
+        return; // Already set, don't overwrite
+    }
 
-    } else if (entity) {
-        free(crm_system_name);
+    if (entity != NULL) {
         crm_system_name = strdup(entity);
 
-    } else if (argc > 0 && argv != NULL) {
+    } else if ((argc > 0) && (argv != NULL)) {
         char *mutable = strdup(argv[0]);
         char *modified = basename(mutable);
 
         if (strstr(modified, "lt-") == modified) {
             modified += 3;
         }
-
-        free(crm_system_name);
         crm_system_name = strdup(modified);
         free(mutable);
 
-    } else if (crm_system_name == NULL) {
+    } else {
         crm_system_name = strdup("Unknown");
     }
 
+    CRM_ASSERT(crm_system_name != NULL);
+
     setenv("PCMK_service", crm_system_name, 1);
 }
 
@@ -692,55 +766,84 @@ crm_log_preinit(const char *entity, int argc, char **argv)
 {
     /* Configure libqb logging with nothing turned on */
 
+    struct utsname res;
     int lpc = 0;
     int32_t qb_facility = 0;
+    pid_t pid = getpid();
+    const char *nodename = "localhost";
+    static bool have_logging = false;
 
-    static bool have_logging = FALSE;
-
-    if(have_logging == FALSE) {
-        have_logging = TRUE;
-
-        crm_xml_init(); /* Sets buffer allocation strategy */
-
-        if (crm_trace_nonlog == 0) {
-            crm_trace_nonlog = g_quark_from_static_string("Pacemaker non-logging tracepoint");
-        }
-
-        umask(S_IWGRP | S_IWOTH | S_IROTH);
-
-        /* Redirect messages from glib functions to our handler */
-        glib_log_default = g_log_set_default_handler(crm_glib_handler, NULL);
-
-        /* and for good measure... - this enum is a bit field (!) */
-        g_log_set_always_fatal((GLogLevelFlags) 0); /*value out of range */
-
-        /* Who do we log as */
-        crm_identity(entity, argc, argv);
-
-        qb_facility = qb_log_facility2int("local0");
-        qb_log_init(crm_system_name, qb_facility, LOG_ERR);
-        crm_log_level = LOG_CRIT;
-
-        /* Nuke any syslog activity until it's asked for */
-        qb_log_ctl(QB_LOG_SYSLOG, QB_LOG_CONF_ENABLED, QB_FALSE);
-#ifdef HAVE_qb_log_conf_QB_LOG_CONF_MAX_LINE_LEN
-        // Shorter than default, generous for what we *should* send to syslog
-        qb_log_ctl(QB_LOG_SYSLOG, QB_LOG_CONF_MAX_LINE_LEN, 256);
-#endif
-
-        /* Set format strings and disable threading
-         * Pacemaker and threads do not mix well (due to the amount of forking)
-         */
-        qb_log_tags_stringify_fn_set(crm_quark_to_string);
-        for (lpc = QB_LOG_SYSLOG; lpc < QB_LOG_TARGET_MAX; lpc++) {
-            qb_log_ctl(lpc, QB_LOG_CONF_THREADED, QB_FALSE);
-#ifdef HAVE_qb_log_conf_QB_LOG_CONF_ELLIPSIS
-            // End truncated lines with '...'
-            qb_log_ctl(lpc, QB_LOG_CONF_ELLIPSIS, QB_TRUE);
-#endif
-            set_format_string(lpc, crm_system_name);
-        }
+    if (have_logging) {
+        return;
     }
+
+    have_logging = true;
+
+    crm_xml_init(); /* Sets buffer allocation strategy */
+
+    if (crm_trace_nonlog == 0) {
+        crm_trace_nonlog = g_quark_from_static_string("Pacemaker non-logging tracepoint");
+    }
+
+    umask(S_IWGRP | S_IWOTH | S_IROTH);
+
+    /* Redirect messages from glib functions to our handler */
+    glib_log_default = g_log_set_default_handler(crm_glib_handler, NULL);
+
+    /* and for good measure... - this enum is a bit field (!) */
+    g_log_set_always_fatal((GLogLevelFlags) 0); /*value out of range */
+
+    /* Set crm_system_name, which is used as the logging name. It may also
+     * be used for other purposes such as an IPC client name.
+     */
+    set_identity(entity, argc, argv);
+
+    qb_facility = qb_log_facility2int("local0");
+    qb_log_init(crm_system_name, qb_facility, LOG_ERR);
+    crm_log_level = LOG_CRIT;
+
+    /* Nuke any syslog activity until it's asked for */
+    qb_log_ctl(QB_LOG_SYSLOG, QB_LOG_CONF_ENABLED, QB_FALSE);
+#ifdef HAVE_qb_log_conf_QB_LOG_CONF_MAX_LINE_LEN
+    // Shorter than default, generous for what we *should* send to syslog
+    qb_log_ctl(QB_LOG_SYSLOG, QB_LOG_CONF_MAX_LINE_LEN, 256);
+#endif
+    if (uname(memset(&res, 0, sizeof(res))) == 0 && *res.nodename != '\0') {
+        nodename = res.nodename;
+    }
+
+    /* Set format strings and disable threading
+     * Pacemaker and threads do not mix well (due to the amount of forking)
+     */
+    qb_log_tags_stringify_fn_set(crm_quark_to_string);
+    for (lpc = QB_LOG_SYSLOG; lpc < QB_LOG_TARGET_MAX; lpc++) {
+        qb_log_ctl(lpc, QB_LOG_CONF_THREADED, QB_FALSE);
+#ifdef HAVE_qb_log_conf_QB_LOG_CONF_ELLIPSIS
+        // End truncated lines with '...'
+        qb_log_ctl(lpc, QB_LOG_CONF_ELLIPSIS, QB_TRUE);
+#endif
+        set_format_string(lpc, crm_system_name, pid, nodename);
+    }
+
+#ifdef ENABLE_NLS
+    /* Enable translations (experimental). Currently we only have a few
+     * proof-of-concept translations for some option help. The goal would be to
+     * offer translations for option help and man pages rather than logs or
+     * documentation, to reduce the burden of maintaining them.
+     */
+
+    // Load locale information for the local host from the environment
+    setlocale(LC_ALL, "");
+
+    // Tell gettext where to find Pacemaker message catalogs
+    CRM_ASSERT(bindtextdomain(PACKAGE, PCMK__LOCALE_DIR) != NULL);
+
+    // Tell gettext to use the Pacemaker message catalogs
+    CRM_ASSERT(textdomain(PACKAGE) != NULL);
+
+    // Tell gettext that the translated strings are stored in UTF-8
+    bind_textdomain_codeset(PACKAGE, "UTF-8");
+#endif
 }
 
 gboolean
@@ -748,7 +851,7 @@ crm_log_init(const char *entity, uint8_t level, gboolean daemon, gboolean to_std
              int argc, char **argv, gboolean quiet)
 {
     const char *syslog_priority = NULL;
-    const char *facility = pcmk__env_option("logfacility");
+    const char *facility = pcmk__env_option(PCMK__ENV_LOGFACILITY);
     const char *f_copy = facility;
 
     pcmk__is_daemon = daemon;
@@ -766,12 +869,12 @@ crm_log_init(const char *entity, uint8_t level, gboolean daemon, gboolean to_std
         if (pcmk__is_daemon) {
             facility = "daemon";
         } else {
-            facility = "none";
+            facility = PCMK__VALUE_NONE;
         }
-        pcmk__set_env_option("logfacility", facility);
+        pcmk__set_env_option(PCMK__ENV_LOGFACILITY, facility);
     }
 
-    if (pcmk__str_eq(facility, "none", pcmk__str_casei)) {
+    if (pcmk__str_eq(facility, PCMK__VALUE_NONE, pcmk__str_casei)) {
         quiet = TRUE;
 
 
@@ -779,13 +882,13 @@ crm_log_init(const char *entity, uint8_t level, gboolean daemon, gboolean to_std
         qb_log_ctl(QB_LOG_SYSLOG, QB_LOG_CONF_FACILITY, qb_log_facility2int(facility));
     }
 
-    if (pcmk__env_option_enabled(crm_system_name, "debug")) {
+    if (pcmk__env_option_enabled(crm_system_name, PCMK__ENV_DEBUG)) {
         /* Override the default setting */
         crm_log_level = LOG_DEBUG;
     }
 
     /* What lower threshold do we have for sending to syslog */
-    syslog_priority = pcmk__env_option("logpriority");
+    syslog_priority = pcmk__env_option(PCMK__ENV_LOGPRIORITY);
     if (syslog_priority) {
         crm_log_priority = crm_priority2int(syslog_priority);
     }
@@ -798,7 +901,7 @@ crm_log_init(const char *entity, uint8_t level, gboolean daemon, gboolean to_std
     }
 
     /* Should we log to stderr */ 
-    if (pcmk__env_option_enabled(crm_system_name, "stderr")) {
+    if (pcmk__env_option_enabled(crm_system_name, PCMK__ENV_STDERR)) {
         /* Override the default setting */
         to_stderr = TRUE;
     }
@@ -806,24 +909,24 @@ crm_log_init(const char *entity, uint8_t level, gboolean daemon, gboolean to_std
 
     // Log to a file if we're a daemon or user asked for one
     {
-        const char *logfile = pcmk__env_option("logfile");
+        const char *logfile = pcmk__env_option(PCMK__ENV_LOGFILE);
 
-        if (!pcmk__str_eq("none", logfile, pcmk__str_casei)
+        if (!pcmk__str_eq(PCMK__VALUE_NONE, logfile, pcmk__str_casei)
             && (pcmk__is_daemon || (logfile != NULL))) {
             // Daemons always get a log file, unless explicitly set to "none"
-            crm_add_logfile(logfile);
+            pcmk__add_logfile(logfile);
         }
     }
 
     if (pcmk__is_daemon
-        && pcmk__env_option_enabled(crm_system_name, "blackbox")) {
+        && pcmk__env_option_enabled(crm_system_name, PCMK__ENV_BLACKBOX)) {
         crm_enable_blackbox(0);
     }
 
     /* Summary */
     crm_trace("Quiet: %d, facility %s", quiet, f_copy);
-    pcmk__env_option("logfile");
-    pcmk__env_option("logfacility");
+    pcmk__env_option(PCMK__ENV_LOGFILE);
+    pcmk__env_option(PCMK__ENV_LOGFACILITY);
 
     crm_update_callsites();
 
@@ -841,7 +944,6 @@ crm_log_init(const char *entity, uint8_t level, gboolean daemon, gboolean to_std
 
     if (pcmk__is_daemon) {
         int user = getuid();
-        const char *base = CRM_CORE_DIR;
         struct passwd *pwent = getpwuid(user);
 
         if (pwent == NULL) {
@@ -850,21 +952,11 @@ crm_log_init(const char *entity, uint8_t level, gboolean daemon, gboolean to_std
         } else if (!pcmk__strcase_any_of(pwent->pw_name, "root", CRM_DAEMON_USER, NULL)) {
             crm_trace("Don't change active directory for regular user: %s", pwent->pw_name);
 
-        } else if (chdir(base) < 0) {
-            crm_perror(LOG_INFO, "Cannot change active directory to %s", base);
+        } else if (chdir(CRM_CORE_DIR) < 0) {
+            crm_perror(LOG_INFO, "Cannot change active directory to " CRM_CORE_DIR);
 
         } else {
-            crm_info("Changed active directory to %s", base);
-#if 0
-            {
-                char path[512];
-
-                snprintf(path, 512, "%s-%lu", crm_system_name, (unsigned long) getpid());
-                mkdir(path, 0750);
-                chdir(path);
-                crm_info("Changed active directory to %s/%s/%s", base, pwent->pw_name, path);
-            }
-#endif
+            crm_info("Changed active directory to " CRM_CORE_DIR);
         }
 
         /* Original meanings from signal(7)
@@ -994,3 +1086,35 @@ crm_log_output_fn(const char *file, const char *function, int line, int level, c
 
     } while (next != NULL && next[0] != 0);
 }
+
+void
+pcmk__cli_init_logging(const char *name, unsigned int verbosity)
+{
+    crm_log_init(name, LOG_ERR, FALSE, FALSE, 0, NULL, TRUE);
+
+    for (int i = 0; i < verbosity; i++) {
+        /* These arguments are ignored, so pass placeholders. */
+        crm_bump_log_level(0, NULL);
+    }
+}
+
+// Deprecated functions kept only for backward API compatibility
+// LCOV_EXCL_START
+
+#include <crm/common/logging_compat.h>
+
+gboolean
+crm_log_cli_init(const char *entity)
+{
+    pcmk__cli_init_logging(entity, 0);
+    return TRUE;
+}
+
+gboolean
+crm_add_logfile(const char *filename)
+{
+    return pcmk__add_logfile(filename) == pcmk_rc_ok;
+}
+
+// LCOV_EXCL_STOP
+// End deprecated API
