@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -51,6 +51,7 @@ pe_free_working_set(pe_working_set_t *data_set)
 {
     if (data_set != NULL) {
         pe_reset_working_set(data_set);
+        data_set->priv = NULL;
         free(data_set);
     }
 }
@@ -69,22 +70,18 @@ pe_free_working_set(pe_working_set_t *data_set)
 gboolean
 cluster_status(pe_working_set_t * data_set)
 {
-    xmlNode *config = get_xpath_object("//"XML_CIB_TAG_CRMCONFIG, data_set->input, LOG_TRACE);
-    xmlNode *cib_nodes = get_xpath_object("//"XML_CIB_TAG_NODES, data_set->input, LOG_TRACE);
-    xmlNode *cib_resources = get_xpath_object("//"XML_CIB_TAG_RESOURCES, data_set->input, LOG_TRACE);
-    xmlNode *cib_status = get_xpath_object("//"XML_CIB_TAG_STATUS, data_set->input, LOG_TRACE);
-    xmlNode *cib_tags = get_xpath_object("//" XML_CIB_TAG_TAGS, data_set->input,
-                                         LOG_NEVER);
-    const char *value = crm_element_value(data_set->input, XML_ATTR_HAVE_QUORUM);
+    xmlNode *section = NULL;
+
+    if ((data_set == NULL) || (data_set->input == NULL)) {
+        return FALSE;
+    }
 
     crm_trace("Beginning unpack");
 
-    /* reset remaining global variables */
-    data_set->failed = create_xml_node(NULL, "failed-ops");
-
-    if (data_set->input == NULL) {
-        return FALSE;
+    if (data_set->failed != NULL) {
+        free_xml(data_set->failed);
     }
+    data_set->failed = create_xml_node(NULL, "failed-ops");
 
     if (data_set->now == NULL) {
         data_set->now = crm_time_new(NULL);
@@ -95,7 +92,7 @@ cluster_status(pe_working_set_t * data_set)
                                                    XML_ATTR_DC_UUID);
     }
 
-    if (crm_is_true(value)) {
+    if (pcmk__xe_attr_is_true(data_set->input, XML_ATTR_HAVE_QUORUM)) {
         pe__set_working_set_flags(data_set, pe_flag_have_quorum);
     } else {
         pe__clear_working_set_flags(data_set, pe_flag_have_quorum);
@@ -106,7 +103,9 @@ cluster_status(pe_working_set_t * data_set)
     data_set->rsc_defaults = get_xpath_object("//" XML_CIB_TAG_RSCCONFIG,
                                               data_set->input, LOG_NEVER);
 
-    unpack_config(config, data_set);
+    section = get_xpath_object("//" XML_CIB_TAG_CRMCONFIG, data_set->input,
+                               LOG_TRACE);
+    unpack_config(section, data_set);
 
    if (!pcmk_any_flags_set(data_set->flags,
                            pe_flag_quick_location|pe_flag_have_quorum)
@@ -114,17 +113,25 @@ cluster_status(pe_working_set_t * data_set)
         crm_warn("Fencing and resource management disabled due to lack of quorum");
     }
 
-    unpack_nodes(cib_nodes, data_set);
+    section = get_xpath_object("//" XML_CIB_TAG_NODES, data_set->input,
+                               LOG_TRACE);
+    unpack_nodes(section, data_set);
 
+    section = get_xpath_object("//" XML_CIB_TAG_RESOURCES, data_set->input,
+                               LOG_TRACE);
     if (!pcmk_is_set(data_set->flags, pe_flag_quick_location)) {
-        unpack_remote_nodes(cib_resources, data_set);
+        unpack_remote_nodes(section, data_set);
     }
+    unpack_resources(section, data_set);
 
-    unpack_resources(cib_resources, data_set);
-    unpack_tags(cib_tags, data_set);
+    section = get_xpath_object("//" XML_CIB_TAG_TAGS, data_set->input,
+                               LOG_NEVER);
+    unpack_tags(section, data_set);
 
     if (!pcmk_is_set(data_set->flags, pe_flag_quick_location)) {
-        unpack_status(cib_status, data_set);
+        section = get_xpath_object("//"XML_CIB_TAG_STATUS, data_set->input,
+                                   LOG_TRACE);
+        unpack_status(section, data_set);
     }
 
     if (!pcmk_is_set(data_set->flags, pe_flag_no_counts)) {
@@ -132,6 +139,9 @@ cluster_status(pe_working_set_t * data_set)
              item = item->next) {
             ((pe_resource_t *) (item->data))->fns->count(item->data);
         }
+        crm_trace("Cluster resource count: %d (%d disabled, %d blocked)",
+                  data_set->ninstances, data_set->disabled_resources,
+                  data_set->blocked_resources);
     }
 
     pe__set_working_set_flags(data_set, pe_flag_have_status);
@@ -150,10 +160,10 @@ cluster_status(pe_working_set_t * data_set)
  * \todo Refactor pe_node_t to strdup() the node name.
  */
 static void
-pe_free_resources(GListPtr resources)
+pe_free_resources(GList *resources)
 {
     pe_resource_t *rsc = NULL;
-    GListPtr iterator = resources;
+    GList *iterator = resources;
 
     while (iterator != NULL) {
         rsc = (pe_resource_t *) iterator->data;
@@ -166,9 +176,9 @@ pe_free_resources(GListPtr resources)
 }
 
 static void
-pe_free_actions(GListPtr actions)
+pe_free_actions(GList *actions)
 {
-    GListPtr iterator = actions;
+    GList *iterator = actions;
 
     while (iterator != NULL) {
         pe_free_action(iterator->data);
@@ -180,7 +190,7 @@ pe_free_actions(GListPtr actions)
 }
 
 static void
-pe_free_nodes(GListPtr nodes)
+pe_free_nodes(GList *nodes)
 {
     for (GList *iterator = nodes; iterator != NULL; iterator = iterator->next) {
         pe_node_t *node = (pe_node_t *) iterator->data;
@@ -220,9 +230,9 @@ pe_free_nodes(GListPtr nodes)
 }
 
 static void
-pe__free_ordering(GListPtr constraints)
+pe__free_ordering(GList *constraints)
 {
-    GListPtr iterator = constraints;
+    GList *iterator = constraints;
 
     while (iterator != NULL) {
         pe__ordering_t *order = iterator->data;
@@ -239,9 +249,9 @@ pe__free_ordering(GListPtr constraints)
 }
 
 static void
-pe__free_location(GListPtr constraints)
+pe__free_location(GList *constraints)
 {
-    GListPtr iterator = constraints;
+    GList *iterator = constraints;
 
     while (iterator != NULL) {
         pe__location_t *cons = iterator->data;
@@ -357,38 +367,36 @@ pe_reset_working_set(pe_working_set_t *data_set)
 void
 set_working_set_defaults(pe_working_set_t * data_set)
 {
+    void *priv = data_set->priv;
+
     memset(data_set, 0, sizeof(pe_working_set_t));
 
+    data_set->priv = priv;
     data_set->order_id = 1;
     data_set->action_id = 1;
     data_set->no_quorum_policy = no_quorum_stop;
 
     data_set->flags = 0x0ULL;
 
-#ifdef DEFAULT_CONCURRENT_FENCING_TRUE
-    pe__set_working_set_flags(data_set,
-                              pe_flag_stop_rsc_orphans
-                              |pe_flag_symmetric_cluster
-                              |pe_flag_stop_action_orphans
-                              |pe_flag_concurrent_fencing);
-#else
     pe__set_working_set_flags(data_set,
                               pe_flag_stop_rsc_orphans
                               |pe_flag_symmetric_cluster
                               |pe_flag_stop_action_orphans);
-#endif
+    if (!strcmp(PCMK__CONCURRENT_FENCING_DEFAULT, "true")) {
+        pe__set_working_set_flags(data_set, pe_flag_concurrent_fencing);
+    }
 }
 
 pe_resource_t *
-pe_find_resource(GListPtr rsc_list, const char *id)
+pe_find_resource(GList *rsc_list, const char *id)
 {
     return pe_find_resource_with_flags(rsc_list, id, pe_find_renamed);
 }
 
 pe_resource_t *
-pe_find_resource_with_flags(GListPtr rsc_list, const char *id, enum pe_find flags)
+pe_find_resource_with_flags(GList *rsc_list, const char *id, enum pe_find flags)
 {
-    GListPtr rIter = NULL;
+    GList *rIter = NULL;
 
     for (rIter = rsc_list; id && rIter; rIter = rIter->next) {
         pe_resource_t *parent = rIter->data;
@@ -404,7 +412,7 @@ pe_find_resource_with_flags(GListPtr rsc_list, const char *id, enum pe_find flag
 }
 
 pe_node_t *
-pe_find_node_any(GListPtr nodes, const char *id, const char *uname)
+pe_find_node_any(GList *nodes, const char *id, const char *uname)
 {
     pe_node_t *match = pe_find_node_id(nodes, id);
 
@@ -416,9 +424,9 @@ pe_find_node_any(GListPtr nodes, const char *id, const char *uname)
 }
 
 pe_node_t *
-pe_find_node_id(GListPtr nodes, const char *id)
+pe_find_node_id(GList *nodes, const char *id)
 {
-    GListPtr gIter = nodes;
+    GList *gIter = nodes;
 
     for (; gIter != NULL; gIter = gIter->next) {
         pe_node_t *node = (pe_node_t *) gIter->data;
@@ -432,9 +440,9 @@ pe_find_node_id(GListPtr nodes, const char *id)
 }
 
 pe_node_t *
-pe_find_node(GListPtr nodes, const char *uname)
+pe_find_node(GList *nodes, const char *uname)
 {
-    GListPtr gIter = nodes;
+    GList *gIter = nodes;
 
     for (; gIter != NULL; gIter = gIter->next) {
         pe_node_t *node = (pe_node_t *) gIter->data;
